
import java.awt.*;

import Optical.Elements.*;
import Optical.Rays.*;
import Optical.Materials.*;
import Optical.Gui.*;

import Utils.*;

public class Achromat extends OpticalApp
{
	OpticalCanvas		canvas;
	OpticalControl		controls;
	OpticalDevice		v;

	Spherical			s1, s2;
	RayCaster			rcRed, rcBlue, rcGreen;
	RayCastersCollection	rcTotal;
	DeviceSwitcher		DS;
	ThickLens			a1, a2;

	Material			BK7, F2, Vaccum, Exa, Exa2;

	static final double	initDeviceX = 20.0,
						initDeviceY = 100.0,
						initLensDistance = 80.0,
						initScreenDistance = 300.0,
						initScreenWidth = 150.0,
						initBeamWidth = 65.0,
						initBeamY = 0.0,
						lambdaRed = 0.6563,
						lambdaGreen = 0.5876,
						lambdaBlue = 0.4861,
						minGap = 0.001;

	static final int	initBeamRays = 7;

	public void init()
	{
		setLayout( new BorderLayout() );
		canvas = new OpticalCanvas();
		BuildDevice();
		canvas.SetDevice( v );

		rcRed = new ParallelRays(
			new FPoint( initDeviceX, initDeviceY + initBeamY, 0.0 ),
			new FPoint( 1.0, 0.0, 0.0 ),
			initBeamRays,
			initBeamWidth,
			lambdaRed,
			Vaccum );
		rcBlue = new ParallelRays(
			new FPoint( initDeviceX, initDeviceY + initBeamY, 0.0 ),
			new FPoint( 1.0, 0.0, 0.0 ),
			initBeamRays,
			initBeamWidth,
			lambdaBlue,
			Vaccum );
		rcGreen = new ParallelRays(
			new FPoint( initDeviceX, initDeviceY + initBeamY, 0.0 ),
			new FPoint( 1.0, 0.0, 0.0 ),
			initBeamRays,
			initBeamWidth,
			lambdaGreen,
			Vaccum );

		rcTotal = new RayCastersCollection();
		rcTotal.Append( rcRed );
		rcTotal.Append( rcBlue );

		canvas.SetRayCaster( rcTotal );
		//canvas.SetRayCaster( rcRed );

		add( "Center", canvas );
		controls = new OpticalControl( this );
		BuildControls();
		add( "South", controls );
	}

	public void BuildDevice()
	{
		BK7 = new Material( "BK7", new Sellmeier1Parameters(
			1.03961212,
			6.00069867e-3,
			2.31792344e-1,
			2.00179144e-2,
			1.01046945,
			1.03560653e-2
 		) );

		F2 = new Material( "F2", new Sellmeier1Parameters(
			1.34533359,
			9.97743871e-3,
			2.09073176e-1,
			4.70450767e-2,
			9.37357162e-1,
			1.11886764e+2
 		) );

		Exa = new Material( "Exa", new ConradyParameters(
			1.3,
			0.1,
			0.0 ) );

		Exa2 = new Material( "Exa2", new ConradyParameters(
			1.1,
			0.2,
			0.0 ) );

		/*System.out.println( "Exa" );
		System.out.println( Exa.AbbeIndex() );
		System.out.println( Exa.AbbeNumber() );
		System.out.println( "Exa2" );
		System.out.println( Exa2.AbbeIndex() );
		System.out.println( Exa2.AbbeNumber() );*/
		//System.out.println( Exa.IndexAtWavelength( 0.6563 ) );

		Vaccum = new Material( "Vaccum", new ConstantParameter( 1.0 ) );

		v = new OpticalDevice();
		v.MoveAxis( initDeviceY, 0.0 );
		v.MoveOnAxis( initDeviceX );
		
		v.Append( new Homogeneous( initLensDistance, Vaccum ) );

		DS = new DeviceSwitcher();

		DS.AddDevice( "1", new ThickLens( - 1/100.0, 1/100.0, 60.0, 50.0,  60.0, Exa, Vaccum ) );

		OpticalDevice temp = new OpticalDevice();
		a1 = new ThickLens( -1/100.0, 1/100.0, 60.0, minGap,  60.0, Exa, Exa2 );
		a2 = new ThickLens(  1/100.0,  0.0,     20.0, 20.0, 60.0, Exa2, Vaccum );
		temp.Append( a1 );
		temp.Append( a2 );

		DS.AddDevice( "2", temp );
		DS.SetCurrentDevice( "1" );
		//DS.Rearrange();

		v.Append( DS );
		//v.Append( temp );

		v.Append( new Homogeneous( initScreenDistance, Vaccum ) );
		v.Append( new Screen( initScreenWidth, initScreenWidth ) );

		v.Rearrange();
	}

	public void BuildControls()
	{
		controls.setLayout( null );
		controls.setSize( 200, 200 );
		
		Button b1 = new Button( "Both" );
		b1.addActionListener( controls );
		b1.setBounds( 20, 20, 80, 20 );
		controls.add( b1 );
		
		b1 = new Button( "Red" );
		b1.addActionListener( controls );
		b1.setBounds( 120, 20, 80, 20 );
		controls.add( b1 );

		b1 = new Button( "Variable" );
		b1.addActionListener( controls );
		b1.setBounds( 20, 50, 80, 20 );
		controls.add( b1 );

		b1 = new Button( "Blue" );
		b1.addActionListener( controls );
		b1.setBounds( 120, 50, 80, 20 );
		controls.add( b1 );

		b1 = new Button( "Normal" );
		b1.addActionListener( controls );
		b1.setBounds( 20, 80, 80, 20 );
		controls.add( b1 );

		b1 = new Button( "Achromat" );
		b1.addActionListener( controls );
		b1.setBounds( 120, 80, 80, 20 );
		controls.add( b1 );

	/*	b1 = new Button( "Print" );
		b1.addActionListener( controls );
		b1.setBounds( 220, 80, 80, 20 );
		controls.add( b1 );
*/
		Scrollbar sb = new Scrollbar(
			Scrollbar.HORIZONTAL,
			486,					// initial
			20,						// thumb
			400,					// min
			657 + 20				// max
			);

		sb.setName( "Lambda" );
		sb.addAdjustmentListener( controls );
		controls.add( sb );
		sb.setBounds( 20, 120, 300, 20 );

		sb = new Scrollbar(
			Scrollbar.HORIZONTAL,
			0,					// initial
			2,						// thumb
			-80,					// min
			80 + 2				// max
			);

		sb.setName( "C3" );
		sb.addAdjustmentListener( controls );
		controls.add( sb );
		sb.setBounds( 20, 150, 300, 20 );

	}

	public void Button( String name )
	{
		if( name.equals( "Both" ) ) canvas.SetRayCaster( rcTotal );
		if( name.equals( "Red" ) ) canvas.SetRayCaster( rcRed );
		if( name.equals( "Variable" ) ) canvas.SetRayCaster( rcGreen );
		if( name.equals( "Blue" ) ) canvas.SetRayCaster( rcBlue );
		if( name.equals( "Normal" ) ) DS.SetCurrentDevice( "1" );
		if( name.equals( "Achromat" ) ) DS.SetCurrentDevice( "2" );
		if( name.equals( "Print" ) )
		{
			System.out.println(
				Exa.IndexAtWavelength( ((ParallelRays)rcGreen).GetWavelength() ) + " " +
				Exa2.IndexAtWavelength( ((ParallelRays)rcGreen).GetWavelength() ) );
		}

		v.Rearrange();
		canvas.ForceRedraw();
	}

	public void Checkbox( String name, boolean value )
	{
		if( value == true )
		{
			canvas.SetRayCaster( rcBlue );
			//DS.SetDevice( "2" );
		}
		else
		{
			canvas.SetRayCaster( rcRed );
			//DS.SetDevice( "1" );
		}
		v.Rearrange();
		canvas.ForceRedraw();
	}


	public void Scrollbar( String name, int value )
	{
		if( name.equals( "Lambda" ) )
		{
			((ParallelRays)rcGreen).SetWavelength( value / 1000.0 );
			canvas.ForceRedraw();
		}
		if( name.equals( "C3" ) )
		{
			double c1, c2, c3;
			double v1, v2, f1, f2, f = 200.0;

			v1 = Exa.AbbeNumber();
			v2 = Exa2.AbbeNumber();

			f1 = f * ( 1 - ( v2 / v1 ) );
			f2 = f * ( 1 - ( v1 / v2 ) );

			c3 = value / 10000.0;
			c2 = 1 / ( f2 * (Exa2.AbbeIndex() - 1) ) + c3;
			c1 = 1 / ( f1 * (Exa.AbbeIndex() - 1) ) + c2;
			a1.SetC1( -c1 );
			a1.SetC2( -c2 );
			a2.SetC1( -c2 );
			a2.SetC2( -c3 );
			v.Rearrange();
			canvas.ForceRedraw();
		}
	}

	public String D2S( double d )
	{
		return Double.toString( Math.round(d*100.0)/100.0 );
	}

	public void destroy()
	{
		remove( canvas );
		remove( controls );
	}

	public void start()
	{
		controls.setEnabled( true );
	}

	public void stop()
	{
		controls.setEnabled( false );
	}
}